import {
  faArrowRightFromBracket,
  faArrowRightToBracket,
  faGear,
} from "@fortawesome/pro-regular-svg-icons";
import { User } from "lucia";
import { Show, createResource } from "solid-js";
import { getRequestEvent } from "solid-js/web";
import { FontAwesomeIcon } from "./FontAwesomeIcon";
import { Li } from "./NavBar";

async function getUser(): Promise<
  | ({
      success: false;
      message: string;
      // user?: undefined;
    } & Partial<User>)
  | ({
      success: true;
      message?: undefined;
    } & User)
> {
  "use server";

  const event = getRequestEvent();
  if (!event) return { success: false, message: "No request event!" };

  const pathname = new URL(event.request.url).pathname;
  const { user } = event.nativeEvent.context;
  if (!user) return { success: false, message: "User not logged in!" };

  console.log("userInfo", pathname, "success");

  return { success: true, ...user };
}

function NavUser() {
  const [user] = createResource(async () => {
    const user = await getUser();

    if (!user.success) console.error("userInfo", user.message);

    return user;
  });
  const pfp = () => {
    const thisUser = user();
    if (!thisUser?.success) return "";

    return thisUser.image
      ? `https://cdn.discordapp.com/avatars/${thisUser.discord_id}/${thisUser.image}.png`
      : `https://cdn.discordapp.com/embed/avatars/${(parseInt(thisUser.discord_id) >> 22) % 6}.png`;
  };

  return (
    <Show
      when={user()?.id}
      fallback={
        <Li href="/api/auth/login" name="Login" rel="external">
          <FontAwesomeIcon
            class="secondary"
            icon={faArrowRightToBracket}
            size="xl"
          />
        </Li>
      }
    >
      <Li href="/config">
        <div class="swap lower">
          <img class="primary" src={pfp()} alt="User pfp" />
          <FontAwesomeIcon class="secondary" icon={faGear} size="xl" />
        </div>
      </Li>
      <Li href="/api/auth/logout" rel="external">
        <FontAwesomeIcon icon={faArrowRightFromBracket} size="xl" />
      </Li>
    </Show>
  );
}

export default NavUser;
