import { DrizzlePostgreSQLAdapter } from "@lucia-auth/adapter-drizzle";
import { Discord } from "arctic";
import { PgColumn, PgTableWithColumns } from "drizzle-orm/pg-core";
import { Lucia } from "lucia";
import db from "~/drizzle";
import { sessions, users } from "~/drizzle/schema";

const adapter = new DrizzlePostgreSQLAdapter(db, sessions, users);

export const lucia = new Lucia(adapter, {
  sessionCookie: {
    attributes: {
      // set to `true` when using HTTPS
      secure: import.meta.env.PROD,
    },
  },
  getUserAttributes: (attributes) => attributes,
});

declare module "lucia" {
  // eslint-disable-next-line no-unused-vars
  interface Register {
    Lucia: typeof lucia;
    DatabaseUserAttributes: DatabaseUserAttributes;
  }
}

type GetColumns<T> =
  T extends PgTableWithColumns<infer First> ? First["columns"] : never;

type ExtractDataTypes<T> = {
  [K in keyof T]: T[K] extends PgColumn<infer DataType, any, any>
    ? DataType["data"]
    : never;
};

interface DatabaseUserAttributes
  extends ExtractDataTypes<GetColumns<typeof users>> {
  warst: string;
}

export const discord = new Discord(
  import.meta.env.VITE_DISCORD_CLIENT_ID,
  import.meta.env.VITE_DISCORD_CLIENT_SECRET,
  import.meta.env.VITE_AUTH_REDIRECT_URL,
);
