import { APIEvent } from "@solidjs/start/server/types";
import { eq } from "drizzle-orm";
import db from "~/drizzle";
import { guilds } from "~/drizzle/schema";

export const GET = async ({ params }: APIEvent) => {
  if (params.guildId === "boot") {
    const guilds = await db.query.guilds
      .findMany({
        with: {
          timePlanning: { with: { messages: true } },
          matches: true,
        },
      })
      .execute();

    return { guilds };
  }

  const guild = await db.query.guilds
    .findFirst({
      where: eq(guilds.id, params.guildId),
      with: {
        timePlanning: { with: { messages: true } },
        matches: true,
      },
    })
    .execute();

  if (!guild)
    return new Response(JSON.stringify({ error: "No such guild found." }), {
      status: 404,
    });

  return guild;
};

export const DELETE = async ({ params }: APIEvent) => {
  const guildQuery = await db.query.guilds
    .findFirst({
      where: eq(guilds.id, params.guildId),
      with: {
        timePlanning: { with: { messages: true } },
        matches: true,
      },
    })
    .execute();

  if (!guildQuery)
    return new Response(JSON.stringify({ error: "No such guild found." }), {
      status: 404,
    });

  const guild = await db
    .delete(guilds)
    .where(eq(guilds.id, params.guildId))
    .returning()
    .execute();

  return guild;
};
